<?php

/*******************************************************************************
Add the given IP to the .htaccess file and the banned IP DB table. This is a 
modified, stripped-down, version of the admin function by a similar name. The 
name was changed to prevent having to check if it was present, for speed.
/******************************************************************************/
function AlterHtaccessFile($ip) {
    if (! filter_var($ip, FILTER_VALIDATE_IP, ['flags' => FILTER_FLAG_IPV4]) ) {
        return false;
    }
    $arrayWrite = array();
    $htaccessfile = '';
    $arrayWrite = GetHtaccessFileContents($htaccessfile);       
    
    if ($arrayWrite) {
        $lastline = ''; 
        $matches = FindInHtaccess($arrayWrite, $lastline);
            
        $line = "deny from " . $ip . PHP_EOL;
        if (! empty($matches)) { //view counter changes already exist
            if (! in_array($line, $arrayWrite)) {
                array_splice($arrayWrite, ($matches[0] - 2), 0, $line);
            } 
        } else {
            $blockArray = [];
            $blockArray[] = PHP_EOL . '# Begin IP deny by View Counter from www.oscommerce-solution.com' . PHP_EOL;
            $blockArray[] = 'order allow,deny' . PHP_EOL;
            $blockArray[] = $line;
            $blockArray[] = 'allow from all' . PHP_EOL;
            $blockArray[] = $lastline;
            $arrayWrite = array_merge($arrayWrite, $blockArray);
        } 

        if (! empty($arrayWrite)) {
             $result = WriteHTAccess($htaccessfile, $arrayWrite);
             DeleteThisIP($ip); //remove IP's from tables no longer needed
             return $result; //ERROR_HTACCESS_WRITE_FAILED;
         }  
    }
    return false;    
}

/*******************************************************************************
Delete the given IP from the IP table and Flag entry
*******************************************************************************/
function DeleteThisIP($ip) {
    tep_db_query("delete from view_counter where ip_number = INET_ATON('" . $ip . "')");
    tep_db_query("delete from view_counter_flags where ip_number = INET_ATON('" . $ip . "')");
}

/*******************************************************************************
Common function to find an entry in the ,htaccess file
*******************************************************************************/
function FindInHtaccess($arrayWrite, &$lastline) {
    $lastline = '# End IP deny by View Counter from www.oscommerce-solution.com' . PHP_EOL;
    $matches = array_keys(preg_grep('/' . $lastline . '/', $arrayWrite));
    if (empty($matches)) { //check again with not new line
        $lastline = '# End IP deny by View Counter from www.oscommerce-solution.com';
        $matches = array_keys(preg_grep('/' . $lastline . '/', $arrayWrite));
        $lastline = '# End IP deny by View Counter from www.oscommerce-solution.com' . PHP_EOL;
    }
    return $matches;
}

/*******************************************************************************
Return the requested item based on the IP to be used on the
create account and new address pages.
/******************************************************************************/
function GetAutoFillArray($item = '') {
    require_once(DIR_FS_CATALOG . 'view_counter/IP2Location.php'); //PRE-PHOENIX ONLY
    $db = new \IP2Location\Database(DIR_FS_CATALOG . 'view_counter/' . VIEW_COUNTER_DATABASE, \IP2Location\Database::FILE_IO);
    $records = $db->lookup($_SERVER['REMOTE_ADDR'], \IP2Location\Database::ALL);
    $addrArray = [];
    $countryID = '';
    
    if ($item == 'country') {
        $countries_query = tep_db_query("select countries_id from countries where countries_iso_code_2 = '" . tep_db_input($records['countryCode']) . "'");
        if (tep_db_num_rows($countries_query)) {
            $country = tep_db_fetch_array($countries_query);
            $countryID = $country['countries_id'];
        }  
    }
    
    $addrArray['city'] = (strpos($records['cityName'], 'This is a') === FALSE ? $records['cityName'] : '');
    $addrArray['state'] = (strpos($records['regionName'], 'This is a') === FALSE ? $records['regionName'] : '');
    $addrArray['zipcode'] = (strpos($records['zipCode'], 'This is a') === FALSE ? $records['zipCode'] : '');
    $addrArray['country'] = $countryID;    
 
    return ($addrArray[$item] ?? null);
 }
 
/*******************************************************************************
Load the .htaccess file into an array
/******************************************************************************/
function GetHtaccessFileContents(&$htaccessfile = '', $name_only = false) {
    $separator = ((substr(DIR_FS_CATALOG, -1) != '/') ? '/' : '');
    $htaccessfile = DIR_FS_CATALOG . $separator . '.htaccess';

    if ($name_only) {  //the contents are not needed so return
        return true;
    }

    if (file_exists($htaccessfile)) {
        $htaccessArray = file($htaccessfile);
        return $htaccessArray;
    }
    return false;
}

/******************************************************
A common function to handle invalid attempts. 
$hacker = false: the spoof message is sent.
$hacker = true:  the hacker message is sent.
******************************************************/
function HandleVCBan($ip, $hacker = false) { 
    
    if (! ($isCust = isset($_SESSION['customers_id']))) {
        $cust_query = tep_db_query("SELECT 1 from view_counter_customers where ip_number = INET_ATON('" . $ip . "') limit 1");
        $isCust = tep_db_num_rows($cust_query);
    }
    if (! $isCust) { //ignore customers

        $was_banned = (VIEW_COUNTER_BAD_BOT_TRAP != 'Both' ? TEXT_VIEW_COUNTER_EMAIL_NO : TEXT_VIEW_COUNTER_EMAIL_YES );
        $msg = ($hacker ? TEXT_VIEW_COUNTER_EMAIL_TRAP_MSG_HACKER : TEXT_VIEW_COUNTER_EMAIL_TRAP_MSG_SPOOF);
        $view_check_query = tep_db_query("select 1 from view_counter_banned where ip_number = INET_ATON('" . $ip . "')");
         
        $from_locn = (VIEW_COUNTER_BAD_BOT_TRAP != 'Email' ? 'Hacker (banned)' : 'Hacker (allowed)');
 
        if (tep_db_num_rows($view_check_query) == 0) { //hasn't been banned yet 

            if (VIEW_COUNTER_BAD_BOT_TRAP != 'Email') {  //then ban or both is set
                $sqlData = array('ip_number' => ip2long(tep_db_input($ip)), 'ignore_status' => 0);
                tep_db_perform('view_counter_banned', $sqlData);  
                AlterHtaccessFile($ip);
            } 
            
            if (VIEW_COUNTER_BAD_BOT_TRAP != 'Ban') {    //then send email. Ban, if set to Both, will be handled above
                $email_sent_query = tep_db_query("select 1 from view_counter_emails where ip_number = inet_aton('" . $ip . "') and sent = '1'");
                if (tep_db_num_rows($email_sent_query) == 0) { //shop owner hasn't been notified of this IP yet so do so
                    $subject = sprintf(TEXT_VIEW_COUNTER_EMAIL_TRAP_SUBJECT, $ip); 
                    $msg = sprintf($msg, $ip, $was_banned, 'http://www.projecthoneypot.org/ip_' . $ip);
                    tep_db_query("insert into view_counter_emails (ip_number, sent) values (INET_ATON('" . $ip . "'), '1')");

                    $vcSendTo = (tep_not_null(VIEW_COUNTER_SEND_EMAILS_TO) ? VIEW_COUNTER_SEND_EMAILS_TO : STORE_OWNER_EMAIL_ADDRESS);
                    $accts = explode(',', $vcSendTo);
                    $accts = array_map('trim', $accts);
                    foreach ($accts as $sendto) {
                        tep_mail(STORE_OWNER, $sendto, $subject, $msg, STORE_OWNER, $vcSendTo);
                    }
                }
            } 
        } else if (VIEW_COUNTER_BAD_BOT_TRAP != 'Ban') {    //has been banned but was the shop owner notified?
            $email_sent_query = tep_db_query("select 1 from view_counter_emails where ip_number = inet_aton('" . $ip . "') and sent = '1'");
            if (tep_db_num_rows($email_sent_query) == 0) { //shop owner hasn't been notified of this IP yet so do so
                $subject = sprintf(TEXT_VIEW_COUNTER_EMAIL_TRAP_SUBJECT, $ip); 
                $msg = sprintf($msg, $ip, $was_banned, 'http://www.projecthoneypot.org/ip_' . $ip);
                tep_db_query("insert into view_counter_emails (ip_number, sent) values (INET_ATON('" . $ip . "'), '1')");

                $vcSendTo = (tep_not_null(VIEW_COUNTER_SEND_EMAILS_TO) ? VIEW_COUNTER_SEND_EMAILS_TO : STORE_OWNER_EMAIL_ADDRESS);
                $accts = explode(',', $vcSendTo);
                $accts = array_map('trim', $accts);
                foreach ($accts as $sendto) {
                    tep_mail(STORE_OWNER, $sendto, $subject, $msg, STORE_OWNER, $vcSendTo);
                }    
            }     
        }      
    }
} 

function IgnoreThisIP($ip) {
    $view_check_query = tep_db_query("select 1 from view_counter_banned where ignore_status = 1 and ip_number = INET_ATON('" . tep_db_input($ip) . "') limit 1");
    if (tep_db_num_rows($view_check_query)) { //this IP is OK so ignore
        return true;
    }
    
    return ((strpos(VIEW_COUNTER_GOOD_IP_LIST, $ip) !== FALSE) ? true : false);
}
 
/*******************************************************************************
Write the altered .htaccess file
/******************************************************************************/
function WriteHTAccess($htaccessfile, $arrayWrite) {
    if (file_exists($htaccessfile)) {
        $fp = fopen($htaccessfile, "w" );
        if ($fp) {
            if (flock($fp, LOCK_EX)) {
                foreach ($arrayWrite as $line) {
                    fputs($fp, $line);      
                }
                 
                fflush($fp);    // release lock
                flock($fp, LOCK_UN); 
            } else {
                return false;
            } 
            
            fclose($fp);
            return true;
        }
    }
    return false;
}